"""MongoDB based message queue using pymongo"""

__version__ = '1.1.0'
__url__ = 'https://github.com/discogs/pymongo-job-queue'
__author__ = 'Andy Craze'
__email__ = 'acraze@discogsinc.com'

from jobqueue import JobQueue



import pymongo
from datetime import datetime
import time


class JobQueue:

    def __init__(self, db, silent=False):
        """ Return an instance of a JobQueue.
        Initialization requires one argument, the database,
        since we use one jobqueue collection to cover all
        sites in an installation/database. The second 
        argument specifies if to print status while waiting
        for new job, the default value is False"""
        self.db = db
        self.silent=silent
        if not self._exists():
            print ('Creating jobqueue collection.')
            self._create()
        self.q = self.db['jobqueue']

    def _create(self, capped=True):
        """ Creates a Capped Collection. """
        # TODO - does the size parameter mean number of docs or bytesize?
        try:
            self.db.create_collection('jobqueue',
                                      capped=capped, max=100000,
                                      size=100000, autoIndexId=True)
        except:
            raise Exception('Collection "jobqueue" already created')

    def _exists(self):
        """ Ensures that the jobqueue collection exists in the DB. """
        return 'jobqueue' in self.db.collection_names()

    def valid(self):
        """ Checks to see if the jobqueue is a capped collection. """
        opts = self.db['jobqueue'].options()
        if opts.get('capped', False):
            return True
        return False

    def next(self):
        """ Runs the next job in the queue. """
        cursor = self.q.find({'status': 'waiting'},
                             tailable=True)
        if cursor:
            row = cursor.next()
            row['status'] = 'done'
            row['ts']['started'] = datetime.now()
            row['ts']['done'] = datetime.now()
            self.q.save(row)
            try:
                return row
            except:
                raise Exception('There are no jobs in the queue')

    def pub(self, data=None):
        """ Publishes a doc to the work queue. """
        doc = dict(
            ts={'created': datetime.now(),
                'started': datetime.now(),
                'done': datetime.now()},
            status='waiting',
            data=data)
        try:
            self.q.insert(doc, manipulate=False)
        except:
            raise Exception('could not add to queue')
        return True

    def __iter__(self):
        """ Iterates through all docs in the queue
            andw aits for new jobs when queue is empty. """
        cursor = self.q.find({'status': 'waiting'}, tailable=True)
        while 1:
            try:
                row = cursor.next()
                try:
                    result = self.q.update({'_id': row['_id'],
                                            'status': 'waiting'},
                                           {'$set': {
                                                'status': 'working',
                                                'ts.started': datetime.now()
                                                }
                                            })
                except OperationFailure:
                    print ('Job Failed!!')
                    continue
                print ('---')
                print ('Working on job:')
                yield row
                row['status'] = 'done'
                row['ts']['done'] = datetime.now()
                self.q.save(row)
            except:
                time.sleep(5)
                if not self.silent:
                    print ('waiting!')

    def queue_count(self):
        """ Returns the number of jobs waiting in the queue. """
        cursor = self.q.find({'status': 'waiting'})
        if cursor:
            return cursor.count()

    def clear_queue(self):
        """ Drops the queue collection. """
        self.q.drop()



from pymongo import MongoClient
from jobqueue import JobQueue
import unittest

host = 'localhost'
port = 27017
pair = '%s:%d' % (host, port)


class TestJobQueue(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        client = MongoClient(host, port)
        client.pymongo_test.jobqueue.drop()
        cls.db = client.pymongo_test

    def test_init(self):
        jq = JobQueue(self.db)
        self.assertTrue(jq.valid())
        self.assertRaises(Exception, jq._create)
        jq.clear_queue()

    def test_valid(self):
        jq = JobQueue(self.db)
        jq.db['jobqueue'].drop()
        jq._create(capped=False)
        self.assertFalse(jq.valid())
        self.assertRaises(Exception, jq._create)
        jq.clear_queue()

    def test_publish(self):
        jq = JobQueue(self.db)
        job = {'message': 'hello world!'}
        jq.pub(job)
        self.assertEquals(jq.queue_count(), 1)
        jq.clear_queue()
        jq.q = None  # erase the queue
        self.assertRaises(Exception, jq.pub, job)

    def test_next(self):
        jq = JobQueue(self.db)
        self.assertRaises(Exception, jq.next)
        job = {'message': 'hello world!'}
        jq.pub(job)
        row = jq.next()
        self.assertEquals(row['data']['message'], 'hello world!')
        jq.clear_queue()

    # def test_iter(self):
    #     jq = JobQueue(self.db)
    #     job = {'message': 'hello world!'}
    #     jq.pub(job)
    #     for job in jq:
    #         if job:
    #             self.assertTrue(True, "Found job")
    #             jq.clear_queue()
    #             return
    #     self.assertEquals(False, "No jobs found!")
    #     jq.clear_queue()


if __name__ == '__main__':
    unittest.main()



from distutils.core import setup
from codecs import open  # To use a consistent encoding
from os import path

# here = path.abspath(path.dirname(__file__))

# # Get the long description from the relevant file
# with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
#     long_description = f.read()

setup(
  name = 'pymjq',
  packages = ['pymjq'], # this must be the same as the name above
  version = '1.2.0',
  description = 'Simple MongoDB based job queue',
  # long_description=long_description,
  license = 'MIT',
  author = 'Andy Craze',
  author_email = 'acraze@discogsinc.com',
  url = 'https://github.com/discogs/pymongo-job-queue', # use the URL to the github repo
  download_url = 'https://github.com/discogs/pymongo-job-queue/tarball/1.2.0',
  keywords = ['queue', 'pymongo', 'mongodb', 'job', 'async', 'worker', 'tail'], # arbitrary keywords
  classifiers = [
    # Indicate who the project is intended for
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Libraries :: Python Modules',

    # Pick your license as you wish (should match "license" above)
    'License :: OSI Approved :: MIT License',

    # Specify the Python versions you support here. In particular, ensure
    # that you indicate whether you support Python 2, Python 3 or both.
    'Programming Language :: Python :: 2.7'
    ]
)
